import datetime
import unittest
from unittest.mock import AsyncMock, patch
from databases import Database

from obspy import UTCDateTime

from geomagio.api.db import MetadataDatabaseFactory
from geomagio.metadata import Metadata, MetadataCategory

class TestMetadataDatabaseFactory(unittest.IsolatedAsyncioTestCase):

    @patch("databases.Database.execute", new_callable=AsyncMock)
    async def test_create_metadata_defaults(self, mock_execute):
        test_data = Metadata(
            category=MetadataCategory.INSTRUMENT,
            created_by="test_metadata.py",
            network="NT",
            station="BDT",
            metadata={
                "type": "FGE",
                "channels": {
                    "U": [{"channel": "U_Volt", "offset": 0, "scale": 313.2}],
                    "V": [{"channel": "V_Volt", "offset": 0, "scale": 312.3}],
                    "W": [{"channel": "W_Volt", "offset": 0, "scale": 312.0}],
                },
                "electronics": {
                    "serial": "E0542",
                    "x-scale": 313.2,
                    "y-scale": 312.3,
                    "z-scale": 312.0,
                    "temperature-scale": 0.01,
                },
                "sensor": {
                    "serial": "S0419",
                    "x-constant": 36958,
                    "y-constant": 36849,
                    "z-constant": 36811,
                },
            },
        )

        db = Database("sqlite:///:memory:")

        await MetadataDatabaseFactory(database=db).create_metadata(test_data)

        # assert data_valid, priority, and status are set to the correct defaults
        expected_values = {
            "category": "instrument",
            "created_by": "test_metadata.py",
            "network": "NT",
            "station": "BDT",
            "metadata": {
                "type": "FGE",
                "channels": {
                    "U": [{"channel": "U_Volt", "offset": 0, "scale": 313.2}],
                    "V": [{"channel": "V_Volt", "offset": 0, "scale": 312.3}],
                    "W": [{"channel": "W_Volt", "offset": 0, "scale": 312.0}],
                },
                "electronics": {
                    "serial": "E0542",
                    "x-scale": 313.2,
                    "y-scale": 312.3,
                    "z-scale": 312.0,
                    "temperature-scale": 0.01,
                },
                "sensor": {
                    "serial": "S0419",
                    "x-constant": 36958,
                    "y-constant": 36849,
                    "z-constant": 36811,
                },
            },
            "data_valid": True,
            "priority": 1,
            "status": "new"
        }

        mock_execute.assert_called_once()
        called_params = mock_execute.call_args.args[0].compile().params

        assert called_params == expected_values

    # @patch("databases.Database.execute", new_callable=AsyncMock)
    # async def test_create_metadata_with_ids(self, mock_execute):
    #     now = datetime.datetime.now(tz=datetime.timezone.utc)
    #     test_data = Metadata(
    #         id=1234,
    #         created_time=now,
    #         metadata_id=5678,
    #         category=MetadataCategory.INSTRUMENT,
    #         created_by="test_metadata.py",
    #         network="NT",
    #         station="BDT",
    #         metadata={
    #             "type": "FGE",
    #             "channels": {
    #                 "U": [{"channel": "U_Volt", "offset": 0, "scale": 313.2}],
    #                 "V": [{"channel": "V_Volt", "offset": 0, "scale": 312.3}],
    #                 "W": [{"channel": "W_Volt", "offset": 0, "scale": 312.0}],
    #             },
    #             "electronics": {
    #                 "serial": "E0542",
    #                 "x-scale": 313.2,
    #                 "y-scale": 312.3,
    #                 "z-scale": 312.0,
    #                 "temperature-scale": 0.01,
    #             },
    #             "sensor": {
    #                 "serial": "S0419",
    #                 "x-constant": 36958,
    #                 "y-constant": 36849,
    #                 "z-constant": 36811,
    #             },
    #         },
    #     )

    #     db = Database("sqlite:///:memory:")

    #     await MetadataDatabaseFactory(database=db).create_metadata(test_data)

    #     # assert id is removed
    #     expected_values = {
    #         "created_time": now,
    #         "category": "instrument",
    #         "created_by": "test_metadata.py",
    #         "network": "NT",
    #         "station": "BDT",
    #         "metadata": {
    #             "type": "FGE",
    #             "channels": {
    #                 "U": [{"channel": "U_Volt", "offset": 0, "scale": 313.2}],
    #                 "V": [{"channel": "V_Volt", "offset": 0, "scale": 312.3}],
    #                 "W": [{"channel": "W_Volt", "offset": 0, "scale": 312.0}],
    #             },
    #             "electronics": {
    #                 "serial": "E0542",
    #                 "x-scale": 313.2,
    #                 "y-scale": 312.3,
    #                 "z-scale": 312.0,
    #                 "temperature-scale": 0.01,
    #             },
    #             "sensor": {
    #                 "serial": "S0419",
    #                 "x-constant": 36958,
    #                 "y-constant": 36849,
    #                 "z-constant": 36811,
    #             },
    #         },
    #         "data_valid": True,
    #         "priority": 1,
    #         "status": "new"
    #     }

    #     mock_execute.assert_called_once()
    #     called_params = mock_execute.call_args.args[0].compile().params

    #     assert called_params == expected_values

    # @patch("databases.Database.execute", new_callable=AsyncMock)
    # async def test_create_metadata_with_metadata_id(self, mock_execute):
    #     test_data = Metadata(
    #         metadata_id=5678,
    #         category=MetadataCategory.INSTRUMENT,
    #         created_by="test_metadata.py",
    #         network="NT",
    #         station="BDT",
    #         metadata={
    #             "type": "FGE",
    #             "channels": {
    #                 "U": [{"channel": "U_Volt", "offset": 0, "scale": 313.2}],
    #                 "V": [{"channel": "V_Volt", "offset": 0, "scale": 312.3}],
    #                 "W": [{"channel": "W_Volt", "offset": 0, "scale": 312.0}],
    #             },
    #             "electronics": {
    #                 "serial": "E0542",
    #                 "x-scale": 313.2,
    #                 "y-scale": 312.3,
    #                 "z-scale": 312.0,
    #                 "temperature-scale": 0.01,
    #             },
    #             "sensor": {
    #                 "serial": "S0419",
    #                 "x-constant": 36958,
    #                 "y-constant": 36849,
    #                 "z-constant": 36811,
    #             },
    #         },
    #     )

    #     db = Database("sqlite:///:memory:")

    #     await MetadataDatabaseFactory(database=db).create_metadata(test_data)

    #     # assert metadata_id is removed on values
    #     expected_values = {
    #         "metadata_id": 5678,
    #         "category": "instrument",
    #         "created_by": "test_metadata.py",
    #         "network": "NT",
    #         "station": "BDT",
    #         "metadata": {
    #             "type": "FGE",
    #             "channels": {
    #                 "U": [{"channel": "U_Volt", "offset": 0, "scale": 313.2}],
    #                 "V": [{"channel": "V_Volt", "offset": 0, "scale": 312.3}],
    #                 "W": [{"channel": "W_Volt", "offset": 0, "scale": 312.0}],
    #             },
    #             "electronics": {
    #                 "serial": "E0542",
    #                 "x-scale": 313.2,
    #                 "y-scale": 312.3,
    #                 "z-scale": 312.0,
    #                 "temperature-scale": 0.01,
    #             },
    #             "sensor": {
    #                 "serial": "S0419",
    #                 "x-constant": 36958,
    #                 "y-constant": 36849,
    #                 "z-constant": 36811,
    #             },
    #         },
    #         "data_valid": True,
    #         "priority": 1,
    #         "status": "new"
    #     }

    #     expected_insert = metadata_table.insert().values(**expected_values)
    #     expected_params = expected_insert.compile().params


    #     mock_execute.assert_called_once()
    #     called_params = mock_execute.call_args.args[0].compile().params

    #     assert called_params == expected_params